package main

import (
	"encoding/json"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path/filepath"

	"kumoly.io/tools/configui/configui"
)

func ListFiles(w http.ResponseWriter, r *http.Request) {
	data, err := json.Marshal(files)
	if err != nil {
		panic(err)
	}
	w.Write(data)
}

func GetFile(w http.ResponseWriter, r *http.Request) {
	name := r.URL.Query().Get("name")
	file, ok := files[name]
	if name == "" || !ok {
		MakeResponse(w, 404, []byte("file not found"))
		return
	}
	data, err := file.Read()
	if err != nil {
		panic(err)
	}
	response, err := json.Marshal(map[string]string{
		"path":   file.Path,
		"name":   file.Alias,
		"action": file.Action,
		"data":   string(data),
	})
	if err != nil {
		panic(err)
	}
	w.Header().Set("Content-Type", "application/json")
	w.Write(response)
}

func PostFile(w http.ResponseWriter, r *http.Request) {
	data, err := ioutil.ReadAll(r.Body)
	r.Body.Close()
	if err != nil {
		panic(err)
	}
	f := configui.File{}
	if err := json.Unmarshal(data, &f); err != nil {
		panic(err)
	}
	file, ok := files[f.Alias]
	if !ok {
		MakeResponse(w, 404, []byte("file not found"))
		return
	}
	if err := file.Write([]byte(f.Data)); err != nil {
		panic(err)
	}
	w.Write([]byte("ok"))
}

func Apply(w http.ResponseWriter, r *http.Request) {
	name := r.URL.Query().Get("name")
	file, ok := files[name]
	if name == "" || !ok {
		MakeResponse(w, 404, []byte("file not found"))
		return
	}
	result, err := file.Do()
	log.Println(err)
	if err != nil {
		panic(err)
	}
	w.Write([]byte(result))
}

func Download(w http.ResponseWriter, r *http.Request) {
	if name := r.URL.Query().Get("name"); name != "" {
		if name == "ConfigUI" {
			data, err := GetConfig()
			if err != nil {
				panic(err)
			}
			w.Header().Set("Content-Disposition", `attachment; filename="ConfigUI.json"`)
			w.Write(data)
			return
		}
		file, ok := files[name]
		if !ok {
			MakeResponse(w, 404, []byte("file not found"))
			return
		}
		data, err := file.Read()
		if err != nil {
			panic(err)
		}
		w.Header().Set("Content-Disposition", `attachment; filename="`+filepath.Base(file.Path)+`"`)
		w.Write(data)
		return
	}
	fs := []string{}
	for _, v := range files {
		fs = append(fs, v.Path)
	}
	if flagConfigPath != "" {
		fs = append(fs, flagConfigPath)
	}
	w.Header().Set("Content-Disposition", `attachment; filename="export.tar.gz"`)
	bundle(w, fs, false)
}

func LoadConfig(w http.ResponseWriter, r *http.Request) {
	if flagNoReconfig {
		panic("system reconfig is disabled")
	}
	data, err := ioutil.ReadAll(r.Body)
	r.Body.Close()
	if err != nil {
		panic(err)
	}
	ftmp, err := configui.ReadConfig(string(data))
	if err != nil {
		panic(err)
	}
	if flagConfigPath != "" {
		info, err := os.Stat(flagConfigPath)
		if err != nil {
			panic(err)
		}
		os.WriteFile(flagConfigPath, data, info.Mode())
	}
	files = configui.GetFileMap(ftmp)
	w.Write([]byte("ok"))
}

func getConfigHandler(w http.ResponseWriter, r *http.Request) {
	data, err := GetConfig()
	if err != nil {
		panic(err)
	}
	w.Write(data)
}

func GetConfig() ([]byte, error) {
	config := []configui.File{}
	for _, f := range files {
		config = append(config, *f)
	}
	return json.MarshalIndent(config, "", "  ")
}
